#!/usr/bin/env python
# -*- coding: utf-8 -*-

import json
import requests
import urllib
import urllib.error
import urllib.request
import os
import datetime
from tempfile import gettempdir


def isEmpty(s):
    if (s is None) or (len(s) <= 0):
        return True
    else:
        return False


class ReportFetcher:
    def __init__(self, logger, args) -> None:
        super().__init__()
        self.logger = logger
        self.args = args

    def export_response_report(self, access_token):
        report_url, reqData, requestHeaders = self.prepare_request(access_token)
        self.downloadReport(report_url, "POST", requestHeaders, reqData)

    def prepare_request(self, access_token):
        site_basic_url = self.args.get("site_basic_url")
        report_url = self.prepare_report_url(site_basic_url)

        request = self.prepare_request_page_and_order()
        request = self.prepare_request_dates(request)
        request = self.prepare_request_language(request)

        reqData = json.dumps(request, indent=None, sort_keys=False)

        requestHeaders = {
            "Content-Type": "application/json",
            "Authorization": "Bearer %s" % access_token,
            "Accept": "application/csv"
        }
        return report_url, reqData, requestHeaders

    def prepare_request_language(self, request):
        request["reasonLang"] = self.args.get("language", "en-US")
        return request

    def prepare_request_dates(self, request):
        request["filter"] = {
            "from": int(self.args.get("start_date").timestamp() * 1000),
            "key": "deployDate",
            "to": int(self.args.get("end_date").timestamp() * 1000),
            "type": "time"
        }
        return request

    def prepare_request_page_and_order(self):
        orders = [{"direction": "ASC", "fieldName": "storeName"}, {"direction": "ASC", "fieldName": "productName"}]
        if not self.args.get("limitRows") is None:
            return {
                "orders": orders,
                "page": {
                    "from": 0,
                    "size": self.args.get("limitRows")
                }
            }

        return {
            "orders": orders
        }

    def prepare_report_url(self, site_basic_url):
        report_url = "%s/%s" % (site_basic_url, "v1/report/exporter/response/report")
        return report_url

    def downloadReport(self, url, verb, headersMap, reqData):
        if headersMap is None:
            headersMap = {}

        dataBytes = None
        if not isEmpty(reqData):
            dataBytes = bytes(reqData, 'utf-8')
            headersMap["Content-Length"] = str(len(dataBytes))

        try:
            # InsecureRequestWarning: Unverified HTTPS request is being made.
            requests.packages.urllib3.disable_warnings()

            self.logger.debug("%s %s" % (verb, url))
            connTimeout = self.args.get("connectTimeout", 15)
            rspTimeout = self.args.get("responseTimeout", 3000)
            rsp = requests.request(verb, url, headers=headersMap,
                data=dataBytes, verify=False, timeout=(connTimeout, rspTimeout))
            statusCode = rsp.status_code
            if (statusCode < 200) or (statusCode >= 400):
                raise urllib.error.HTTPError(url, statusCode, "Failed: %d" % statusCode, rsp.headers, None)

            filePath = self.resolveOutputFilePath()
            text_file = open(filePath, "wb")
            for chunk in rsp.iter_content(chunk_size=1024):
                text_file.write(chunk)

            text_file.close()
        except urllib.error.HTTPError as err:
            self.logger.error("Failed (%d %s) to invoke %s %s" % (err.code, err.msg, verb, url))
            raise err
        except urllib.error.URLError as err:
            self.logger.error("Some unknown error for %s %s: %s" % (verb, url, err.reason))
            raise err

    def resolveOutputFilePath(self):
        filePath = self.args.get("file", None)
        if not isEmpty(filePath):
            filePath = self.resolvePathVariables(filePath)
            if os.path.isabs(filePath):
                print("report exported: {}".format(filePath))
                return filePath

        dirPath = self.args.get("dir", None)
        if not isEmpty(dirPath):
            dirPath = self.resolvePathVariables(dirPath)
        else:
            dirPath = gettempdir()


        if isEmpty(filePath):
            today = datetime.datetime.utcnow()
            filePath = "response_report_%s.csv" % today.strftime("%Y%m%d_%H%M")


        if not os.path.isabs(dirPath):
            dirPath = os.path.abspath(dirPath)

        path = os.path.join(dirPath, filePath)
        print("Report exported to: {}".format(path))
        return path

    def resolvePathVariables(self, path):
        """
        Expands ~/xxx and ${XXX} variables
        """
        if isEmpty(path):
            return path

        path = os.path.expanduser(path)
        path = os.path.expandvars(path)
        return path



#!/usr/bin/env python
# -*- coding: utf-8 -*-


import sys
import signal
import os

from reports_exporter.response_report_fetcher import ReportFetcher
from reports_exporter import arguments_parser
from reports_exporter import access_token_fetcher
from reports_exporter.custom_logging import LogFactory


def isEmpty(s):
    if (s is None) or (len(s) <= 0):
        return True
    else:
        return False


def die(msg=None,rc=1):
        """
        Cleanly exits the program with an error message
        """

        if msg:
            sys.stderr.write(msg)
            sys.stderr.write("\n")
            sys.stderr.flush()

        sys.exit(rc)


class ResultFetcher:
    def die(self, msg=None, rc=1):
        """
        Cleanly exits the program with an error message
        """

        if msg:
            sys.stderr.write(msg)
            sys.stderr.write("\n")
            sys.stderr.flush()

        sys.exit(rc)

    def signal_handler(self):
        self.die('Exit due to Control+C')

    def run(self, opt_args=None):
        pyVersion = sys.version_info
        if pyVersion.major != 3:
            self.die("Major Python version must be 3.x: %s" % str(pyVersion))
        if pyVersion.minor < 0:
            self.die("Minor Python version %s should be at least 3.0+" % str(pyVersion))

        signal.signal(signal.SIGINT, self.signal_handler)
        if os.name == 'nt':
            sys.stderr.write("Use Ctrl+Break to stop the script\n")
        else:
            sys.stderr.write("Use Ctrl+C to stop the script\n")

        if opt_args is None:
            argParser = arguments_parser.ArgumentParser()
            args = argParser.parse_arguments()
        else:
            args = opt_args
            arguments_parser.process_args(args)
        self.main(args)

    def main(self, args):
        logFactory = LogFactory(args)
        logger = logFactory.getLogger("main")
        token_fetcher = access_token_fetcher.AccessTokenFetcher(logger, args)
        token = token_fetcher.get_access_token()
        if isEmpty(token):
            die("Could not get access token for the username")
        exporter = ReportFetcher(logger, args)
        logger.info("Started exporting report, this may take a few minutes")
        try:
            exporter.export_response_report(token)
        finally:
            token_fetcher.logout(token)



import sys
import urllib
import urllib.error
import urllib.request
import uuid
import requests
from getpass import getpass



def die(msg=None,rc=1):
        """
        Cleanly exits the program with an error message
        """

        if msg:
            sys.stderr.write(msg)
            sys.stderr.write("\n")
            sys.stderr.flush()

        sys.exit(rc)


def isEmpty(s):
    if (s is None) or (len(s) <= 0):
        return True
    else:
        return False


def buildUrlQueryParams(queryParams):
    if isEmpty(queryParams):
        return None

    nvPairs = []
    for key, value in queryParams.items():
        if isinstance(value, (int, float, bool)):
            value = str(value)
        nvPairs.append("%s=%s" % (key, value))

    return "&".join(nvPairs)


class AccessTokenFetcher:
    def __init__(self, logger, args) -> None:
        super().__init__()
        self.logger = logger
        self.args = args

    def get_access_token(self) -> str:
        password = self.args.get("password")
        if isEmpty(password):
            password = getpass('Password: ')
        return self.resolve_user_token(self.args["username"], password)

    def resolve_user_token(self, username, password):
        self.logger.debug("Retrieving Keycloak access token")

        if isinstance(username, (int, float)):
            username = str(username)
        elif isEmpty(username):
            die("No Keycloak access username provided")

        if isinstance(password, (int, float)):
            password = str(password)
        elif isEmpty(password):
            die("No Keycloak access password provided")

        clientId = self.args.get("clientId", None)

        if isEmpty(clientId):
            die("No Keycloak client identifier provided")

        clientIdFormat = self.args.get("clientIdFormat", "ups-installation-%s")
        effectiveClientId = clientIdFormat % clientId

        mode = self.args.get("mode", "STANDARD")
        mode = mode.upper()
        if mode == "STANDARD":
            rsp = self.resolveStandardUserToken(
                username, password, effectiveClientId)
        elif mode == "DIRECT":
            rsp = self.resolveDirectGrantUserToken(
                username, password, effectiveClientId)
        else:
            die("Unknown Keycloak access token retrieval mode: %s" % mode)

        accessToken = rsp.get("access_token", None)
        self.refresh_token = rsp.get("refresh_token", None)
        if isEmpty(accessToken):
            self.logger.error("No access token in Keycloak response: %s" % str(rsp))
            die("No access token returned from Keycloak")

        self.logger.debug("Retrieved Keycloak access token")
        return accessToken

    def resolveStandardUserToken(self, username, password, clientId):
        params = self.resolveStandardUserTokenRequestParameters(clientId)
        redirectUri = urllib.parse.unquote(params['redirectUri'])

        params = self.resolveKeycloakSessionAuthenticationCode(username, password, params)

        code = params.get("code", None)
        if isEmpty(code):
            die("No Keycloak access token session code")

        requestHeaders = {
            "Accepts": "application/json",
            "Content-Type": "application/x-www-form-urlencoded"
        }

        formData = "&".join(
            (
                "grant_type=authorization_code",
                "code=%s" % code,
                "client_id=%s" % clientId,
                "redirect_uri=%s"% redirectUri
            )
        )

        url = self.resolveKeycloakDirectTokenAccessUrl()
        rsp = self.executeHttpRequest(url, "POST", requestHeaders, formData, asJson=True)
        return rsp["body"]

    def resolveDirectGrantUserToken(self, username, password, clientId):
        # See https://developer.mozilla.org/en-US/docs/Web/HTTP/Methods/POST
        formData = "&".join(
            (
                "grant_type=password",
                "username=%s" % username,
                "password=%s" % password,
                "client_id=%s" % clientId
            )
        )

        requestHeaders = {
            "Accepts": "application/json",
            "Content-Type": "application/x-www-form-urlencoded"
        }

        url = self.resolveKeycloakDirectTokenAccessUrl()
        rsp = self.executeHttpRequest(url, "POST", requestHeaders, formData)
        return rsp["body"]

    def resolveStandardUserTokenRequestParameters(self, clientId):
        params = {}

        baseUrl = self.resolveKeycloakOpenidAccessUrl() + "/auth"
        locItems = urllib.parse.urlparse(baseUrl)
        ## By default, the quote function is intended for quoting the path
        ##  section of a URL.  Thus, it will not encode '/' - unless we override
        ## the 'safe' string
        redirectUri = urllib.parse.quote(locItems.scheme + "://" + locItems.netloc + "/", safe='')
        params["redirectUri"] = redirectUri

        queryParams = {
            "client_id": clientId,
            "redirect_uri": redirectUri,
            "response_mode": "fragment",
            "response_type": "code",
            "scope": "openid",
            "state": str(uuid.uuid4()),
            "nonce": str(uuid.uuid4())
        }

        url = baseUrl + "?" + buildUrlQueryParams(queryParams)
        params["referer"] = url

        rsp = self.executeHttpRequest(url, "GET", { "Accepts": "*/*" }, None, asJson=False)
        params['loginUrl'] = self.extractKeycloakLoginActionValue(rsp['body'].split('\n'))

        cookies = rsp['cookies']
        params['AUTH_SESSION_ID'] = cookies.get("AUTH_SESSION_ID", None)
        params['KC_RESTART'] = cookies.get("KC_RESTART", None)

        return params

    # Returns { "statusCode": ..., "headers": ..., "body": ..., "cookies": ..., "history": ... } object
    def executeHttpRequest(self, url, verb, headersMap, reqData, cookieJar=None, asJson=True):
        if headersMap is None:
            headersMap = {}

        dataBytes = None
        if not isEmpty(reqData):
            dataBytes = bytes(reqData, 'utf-8')
            headersMap["Content-Length"] = str(len(dataBytes))

        try:
            # InsecureRequestWarning: Unverified HTTPS request is being made.
            requests.packages.urllib3.disable_warnings()

            self.logger.debug("%s %s" % (verb, url))
            connTimeout = self.args.get("connectTimeout", 15)
            rspTimeout = self.args.get("responseTimeout", 30)
            rsp = requests.request(verb, url, headers=headersMap,
                data=dataBytes, cookies=cookieJar, verify=False, timeout=(connTimeout, rspTimeout))
            statusCode = rsp.status_code
            if asJson and (statusCode >= 200) and (statusCode < 300):
                rspContent = rsp.json()
            else:
                rspContent = rsp.text
            # NOTE: we extract the response context regardless of the status code
            # so we can place a debug breakpoint here and see it
            if (statusCode < 200) or (statusCode >= 400):
                raise urllib.error.HTTPError(url, statusCode, "Reason: %s. URL: %s" % (rsp.reason, url), rsp.headers, None)
            result = {
                "statusCode": statusCode,
                "headers": rsp.headers,
                "cookies": rsp.cookies,
                "history": rsp.history,
                "body": rspContent
            }
            return result
        except urllib.error.HTTPError as err:
            self.logger.error("Failed (%d %s) to invoke %s %s" % (err.code, err.msg, verb, url))
            raise err
        except urllib.error.URLError as err:
            self.logger.error("Some unknown error for %s %s: %s" % (verb, url, err.reason))
            raise err

    def resolveKeycloakSessionAuthenticationCode(self, username, password, params):
        url = params['loginUrl']
        if isEmpty(url):
            die("Failed to extract Keycloak login URL value")

        cookies = {}

        sessionId = params.get('AUTH_SESSION_ID', None)
        if isEmpty(sessionId):
            die("No Keycloak session ID cookie present")
        cookies["AUTH_SESSION_ID"] = sessionId

        kcToken = params.get('KC_RESTART', None)
        if isEmpty(kcToken):
            die("No Keycloak restart cookie present")
        cookies["KC_RESTART"] = kcToken

        requestHeaders = {
            "Content-Type": "application/x-www-form-urlencoded",
            "Referer": params["referer"]
        }

        formData = "&".join(("username=%s" % username, "password=%s" % password))

        rsp = self.executeHttpRequest(url, "POST", requestHeaders, formData, cookieJar=cookies, asJson=False)
        redirectHistory = rsp.get("history", None)
        if isEmpty(redirectHistory):
            if "INVALID_USER_PASS" in rsp.get("body", None):
                die("Invalid Username or Password")
            else:
                die("No Keycloak redirection available")

        rdData = redirectHistory[0]
        rdHeaders = rdData.headers
        locHeader = rdHeaders.get("Location", None)
        locItems = urllib.parse.urlparse(locHeader)
        # Response is https://..../#state=...
        queryParams = urllib.parse.parse_qs(locItems.fragment)
        code = queryParams.get("code", None)
        if isinstance(code, list):
            code = code[0]

        return {
            "url": rdData.url,
            "cookies": rsp.get("cookies", None),
            "code": code
        }

    def resolveKeycloakDirectTokenAccessUrl(self):
        return self.resolveKeycloakOpenidAccessUrl() + "/token"

    def resolveKeycloakOpenidAccessUrl(self):
        return self.resolveKeycloakRealmAccessUrl() + "/protocol/openid-connect"

    def resolveKeycloakRealmAccessUrl(self):
        if self.args is None:
            die("No Keycloak access arguments provided")

        protocol = self.args.get("protocol", "https")
        host = self.args.get("host", None)
        if isEmpty(host):
            die("No Keycloak host specified")

        port = self.args.get("port", -1)
        realm = self.args.get("realm", "unifiedpush-installations")
        if port > 0:
            return "%s://%s:%d/auth/realms/%s" % (protocol, host, port, realm)
        else:
            return "%s://%s/auth/realms/%s" % (protocol, host, realm)

    def logout(self, accessToken):
        url = self.resolveKeycloakOpenidAccessUrl() + "/logout"
        clientId = self.args.get("clientId", None)
        requestHeaders = {
            "Authorization": "Bearer %s" % accessToken,
            "Content-Type": "application/x-www-form-urlencoded"
        }

        clientIdFormat = self.args.get("clientIdFormat", "ups-installation-%s")
        effectiveClientId = clientIdFormat % clientId

        payload = "&".join(
            (
                "client_id=%s" % effectiveClientId,
                "refresh_token=%s" % self.refresh_token
            )
        )

        self.executeHttpRequest(url, "POST", requestHeaders, payload, asJson=False)

    def extractKeycloakLoginActionValue(self, lines):
        for l in lines:
            pos = l.find("kcLoginAction")
            if pos < 0:
                continue

            pos = l.find("value=")
            if pos < 0:
                continue

            l = l[pos + 6:]
            pos = l.find(' ')
            l = l[0:pos].strip()
            l = l.replace("&amp;", "&")
            return l

        return None



import sys
import datetime
import threading
import traceback


def die(msg=None,rc=1):
        """
        Cleanly exits the program with an error message
        """

        if msg:
            sys.stderr.write(msg)
            sys.stderr.write("\n")
            sys.stderr.flush()

        sys.exit(rc)


class LogLevel(object):
    def __init__(self, name, value):
        self.levelName = name
        self.levelValue = value

    @property
    def name(self):
        return self.levelName

    @name.setter
    def name(self, value):
        raise NotImplementedError("Log level name is immutable")

    @property
    def value(self):
        return self.levelValue

    @value.setter
    def value(self, number):
        raise NotImplementedError("Log level value is immutable")

    def __str__(self):
        return "%s[%s]" % (self.name % str(self.value))


class Logger(object):
    # Because of some Python limitations we need to define these here
    OFF = LogLevel('OFF', 10000)
    ERROR = LogLevel('ERROR', 1000)
    WARNING = LogLevel('WARNING', 900)
    INFO = LogLevel('INFO', 800)
    DEBUG = LogLevel('DEBUG', 700)
    TRACE = LogLevel('TRACE', 600)
    ALL = LogLevel('ALL', 0)
    LEVELS = [OFF, ERROR, WARNING, INFO, DEBUG, TRACE, ALL]

    def __init__(self, name, threshold, args):
        self.loggerName = name
        self.thresholdLevel = threshold
        self.dateTimeFormat = args.get("log-datetime-format", "%Y-%m-%d %H:%M:%S.%f")
        self.maxStackTraceDepth = args.get("log-stacktrace-depth", 10)

    @property
    def name(self):
        return self.loggerName

    @name.setter
    def name(self,value):
        raise NotImplementedError("Logger name is immutable")

    @property
    def threshold(self):
        return self.thresholdLevel

    @threshold.setter
    def threshold(self, value):
        raise NotImplementedError("Not allowed to modify threshold level for %s" % self.name)

    @property
    def errorEnabled(self):
        return self.levelEnabled(Logger.ERROR)

    @errorEnabled.setter
    def errorEnabled(self, value):
        raise NotImplementedError("Not allowed to modify ERROR enabled state for %s" % self.name)

    def error(self, msg, err=None):
        self.log(Logger.ERROR, msg, err)

    @property
    def warningEnabled(self):
        return self.levelEnabled(Logger.WARNING)

    @warningEnabled.setter
    def warningEnabled(self, value):
        raise NotImplementedError("Not allowed to modify WARNING enabled state for %s" % self.name)

    def warning(self, msg, err=None):
        self.log(Logger.WARNING, msg, err)

    @property
    def infoEnabled(self):
        return self.levelEnabled(Logger.INFO)

    @infoEnabled.setter
    def infoEnabled(self, value):
        raise NotImplementedError("Not allowed to modify INFO enabled state for %s" % self.name)

    def info(self, msg, err=None):
        self.log(Logger.INFO, msg, err)

    @property
    def debugEnabled(self):
        return self.levelEnabled(Logger.DEBUG)

    @debugEnabled.setter
    def debugEnabled(self, value):
        raise NotImplementedError("Not allowed to modify DEBUG enabled state for %s" % self.name)

    def debug(self, msg, err=None):
        self.log(Logger.DEBUG, msg, err)

    @property
    def traceEnabled(self):
        return self.levelEnabled(Logger.TRACE)

    @traceEnabled.setter
    def traceEnabled(self, value):
        raise NotImplementedError("Not allowed to modify TRACE enabled state for %s" % self.name)

    def trace(self, msg, err=None):
        self.log(Logger.TRACE, msg, err)

    def log(self, level, msg, err=None):
        if self.levelEnabled(level):
            self.appendLog(level, msg, err)

    def appendLog(self, level, msg, err=None):
        nowValue = datetime.datetime.now()
        timestamp = nowValue.strftime(self.dateTimeFormat)
        threadName = "unknown"
        thread = threading.current_thread()
        if not thread is None:
            threadName = thread.name

        if msg:
            if '\n' in msg:
                for line in msg.splitlines(False):
                    self.writeLogMessage(level, "%s %s [%s] [%s] %s" % (timestamp, threadName, level.name, self.name, line))
            else:
                self.writeLogMessage(level, "%s %s [%s] [%s] %s" % (timestamp, threadName, level.name, self.name, msg))
        if err:
            self.writeLogMessage(level, "%s %s [%s] [%s] %s: %s" % (timestamp, threadName, level.name, self.name, err.__class__.__name__, str(err)))

            if self.maxStackTraceDepth > 0:
                # TODO this doesn't quite do the job - by the time it is here, most stack trace data is gone...
                traceValue = traceback.format_exc(self.maxStackTraceDepth)
                lines = traceValue.splitlines()
                for traceLine in lines:
                    self.writeLogMessage(level, "%s %s [%s] %s %s" % (timestamp, threadName, level.name, self.name, traceLine))

            if err.__class__.__name__ == "KeyboardInterrupt":
                die("Killed by Control+C")

    def writeLogMessage(self, level, msg):
        raise NotImplementedError("%s#writeLogMessage(%s) not implemented" % (self.__class__.__name__, msg))

    def levelEnabled(self, level):
        if not level or not self.threshold:
            return False
        elif level.value >= self.threshold.value:
            return True
        else:
            return False

    def __str__(self):
        return self.name

    @staticmethod
    def fromLevelName(name):
        if (not name) or (len(name) <= 0):
            return None

        effectiveName = name.upper()
        for level in Logger.LEVELS:
            if level.name == effectiveName:
                return level

        return None

    @staticmethod
    def logHelp(logger, lines):
        if (not lines) or (len(lines) <= 0):
            return

        for line in lines:
            logger.info(line)


class StreamLogger(Logger):
    def __init__(self, name, threshold, args):
        super(StreamLogger, self).__init__(name, threshold, args)
        self.targetStream = None
        self.autoFlush = args.get("log-auto-flush", True)

    def writeLogMessage(self, level, msg):
        self.targetStream.write("%s\n" % msg)
        if self.autoFlush:
            self.targetStream.flush()


class ConsoleLogger(StreamLogger):
    def __init__(self, name, threshold, args):
        super(ConsoleLogger, self).__init__(name, threshold, args)
        target = args.get("log-console-target", "stderr").lower()
        if target == "stdout":
            self.targetStream = sys.stdout
        else:
            self.targetStream = sys.stderr


class LogFactory(object):
    def __init__(self, args):
        self.args = args
        self.threshold = Logger.fromLevelName(args.get("log-threshold", Logger.INFO.name))
        if self.threshold is None:
            self.threshold = Logger.OFF

    def getLogger(self, name):
        # TODO add support for more logger types
        return ConsoleLogger(name, self.threshold, self.args)


from setuptools import setup

setup(name='cb4_reports_exporter',
      version='1.0.4',
      description='Export reports from CB4',
      author='CB4',
      author_email="support@cb4.com",
      packages=['reports_exporter'],
      python_requires='>=3.7',
      zip_safe=False)



import argparse
import datetime
import sys


DEFAULT_WEEKS_COUNT = 4
DEFAULT_LOG_THRESHOLD = "INFO"
DEFAULT_FIRST_DAY_OF_WEEK = "MON"
DEFAULT_DOMAIN="mcs.c-b4.com"

SHIFT_DAYS = {
    "SUN": 1,
    "MON": 0,
    "TUE": -1,
    "WED": -2,
    "THU": -3,
    "FRI": -4,
    "SAT": -5,
}


def die(msg=None,rc=1):
    """
    Cleanly exits the program with an error message
    """

    if msg:
        sys.stderr.write(msg)
        sys.stderr.write("\n")
        sys.stderr.flush()

    sys.exit(rc)


def shift_date_to_first_day_of_week(date, day_of_week):
    first_day = date - datetime.timedelta(days=(date.weekday() + SHIFT_DAYS[day_of_week]) % 7)
    return datetime.datetime.combine(first_day, datetime.time.min)


def shift_date_to_end_day_of_week(date, day_of_week):
    end_day = shift_date_to_first_day_of_week(date, day_of_week) + datetime.timedelta(days=6)
    return datetime.datetime.combine(end_day, datetime.time.max)


def isEmpty(s):
    if (s is None) or (len(s) <= 0):
        return True
    else:
        return False


def parse_date(date_str):
    try:
        return datetime.datetime.strptime(date_str, "%Y-%m-%d")
    except:
        raise Exception("Invalid start_date. Format expected: yyyy-mm-dd")


def parse_site_url(args):
    url = args["site_basic_url"]

    protocol = url.split("://")[0]
    args["protocol"] = protocol

    base_url = url.split("://")[1]
    args["host"] = base_url.split(":")[0]
    if len(base_url.split(":")) > 1:
        args["port"] = base_url.split(":")[1]
    domain = args.get("domain")

    if domain not in base_url:
        die("domain %s is missing from site_basic_url %s" % (domain, url))
    client_id = base_url.split("-%s" % domain)[0]
    args["clientId"] = client_id

def manage_start_and_end_dates(args):
    req_start_date = args.get("start_date")
    req_end_date = args.get("end_date")
    day_of_week = args.get("first_day_of_week", DEFAULT_FIRST_DAY_OF_WEEK)
    weeks_count = args.get("weeks_count", DEFAULT_WEEKS_COUNT)
    if isEmpty(req_end_date) and isEmpty(req_start_date):
        today = datetime.datetime.utcnow().replace(hour=0, minute=0, second=0, microsecond=0)
        end_date = shift_date_to_end_day_of_week(today, day_of_week)
        args["end_date"] = end_date
        args["start_date"] = shift_date_to_first_day_of_week(end_date - datetime.timedelta(weeks=weeks_count), day_of_week)
        return

    if not isEmpty(req_end_date) and not isEmpty(req_start_date):
        args["start_date"] = shift_date_to_first_day_of_week(parse_date(req_start_date), day_of_week)
        args["end_date"] = shift_date_to_end_day_of_week(parse_date(req_end_date), day_of_week)
        return

    if not isEmpty(req_start_date):
        start_date = shift_date_to_first_day_of_week(parse_date(req_start_date), day_of_week)
        args["start_date"] = start_date
        args["end_date"] = shift_date_to_end_day_of_week(start_date + datetime.timedelta(weeks=weeks_count), day_of_week)
        return

    if not isEmpty(req_end_date):
        end_date = shift_date_to_end_day_of_week(parse_date(req_end_date), day_of_week)
        args["end_date"] = end_date
        args["start_date"] = shift_date_to_first_day_of_week(end_date - datetime.timedelta(weeks=weeks_count), day_of_week)


def process_args(args):
    password = args.get("password")
    args["domain"] = args.get("domain", DEFAULT_DOMAIN)
    parse_site_url(args)
    manage_start_and_end_dates(args)
    args["log-threshold"] = args.get("log-threshold", DEFAULT_LOG_THRESHOLD)


class ArgumentParser:

    def parse_arguments(self):
        parser = argparse.ArgumentParser(description='Dumps the responses to a CSV file', formatter_class=argparse.RawTextHelpFormatter)
        parser.add_argument('--username',
                            type=str,
                            dest='username',
                            help='login username'
                            )
        parser.add_argument('--password',
                            type=str,
                            dest='password'
                            )
        parser.add_argument('--dir',
                            type=str,
                            dest='dir',
                            help='The CSVs directory path to dump the response report CSV (the default location is the script current directory)'
                            )
        parser.add_argument('--file',
                            type=str,
                            dest='file',
                            help='The CSVs directory path to dump the response report CSV (can be relative to dir or absolute path)'
                            )
        parser.add_argument('--site_basic_url',
                            type=str,
                            dest='site_basic_url',
                            help='Client url, example: https://someClient-mcs.com'
                            )
        parser.add_argument('--limitRows',
                            type=int,
                            dest='limitRows',
                            help='Limit the numbers of rows to export'
                            )
        parser.add_argument('--start_date',
                            type=str,
                            dest='start_date',
                            help='Example: 2020-01-01 (format: YYYY-MM-dd)'
                            )
        parser.add_argument('--end_date',
                            type=str,
                            dest='end_date',
                            help='Example: 2020-07-01 (format: YYYY-MM-dd)'
                            )
        parser.add_argument('--language',
                            type=str,
                            dest='language',
                            default="en-US",
                            help='reasons language. Example: en-US'
                            )
        parser.add_argument('--first_day_of_week',
                            type=str,
                            dest='first_day_of_week',
                            default=DEFAULT_FIRST_DAY_OF_WEEK,
                            help='options: [SUN MON TUE WED THU FRI SAT] , default: MON'
                            )
        parser.add_argument('--weeks_count',
                            type=int,
                            dest='weeks_count',
                            default=DEFAULT_WEEKS_COUNT,
                            help="""If START_DATE & END_DATE are not set -> timespan = (today - weeks_count, today).
If  only START_DATE -> timespan = (start_date, start_date + weeks_count).
If  only only END_DATE ->  timespan = (end_date - weeks_count, end_date).
If both START_DATE & END_DATE are set - WEEKS_COUNT is ignored. 
"""
                            )
        parser.add_argument('--log-threshold',
                            type=str,
                            dest='log-threshold',
                            default=DEFAULT_LOG_THRESHOLD,
                            help="OFF, ERROR, WARNING, INFO, DEBUG, TRACE, ALL"
                            )
        parser.add_argument('--domain',
                            type=str,
                            dest='domain',
                            default=DEFAULT_DOMAIN,
                            help="domain name. default value %s" % DEFAULT_DOMAIN
                            )

        args = parser.parse_args()
        args = vars(args)
        for arg in args:
            if isEmpty(arg):
                args.pop(arg, None)
        process_args(args)
        return args
